(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

open Misc
open Printf

(* Configuration *)
let arch = ref PPC
let name = ref None

let () = Config.nprocs := 1000

let opts =
  Config.common_specs @
  ("-name",
   Arg.String (fun n -> name := Some n),
   "<name> generate test <name> in file <name>.litmus")::
  Util.arch_opt (fun a -> arch := a) (pp_arch !arch)




module Make(C:XXXCompile.S) =
  struct
    module M = Top.Make(C)

    let dump name es  = match name with
    | None ->
        let t = M.make_test "a" es in
        M.dump_test_channel stdout t
    | Some name ->
        let t = M.make_test name es in
        Misc.output_protect
          (fun chan -> M.dump_test_channel chan t)
          (name ^ ".litmus")

    let zyva name pp_rs =
      try
        let pp_rs = List.map LexUtil.split pp_rs in
        let pp_rs = List.concat pp_rs in
        let rs = List.map C.R.parse_relax pp_rs in
        let es =
          List.fold_right
            (fun r k -> C.R.edges_of r @ k)
            rs [] in
        dump name es
      with Fatal msg -> Warn.warn_always "%s" msg
  end

let pp_es = ref []

let () =
  Util.parse_cmdline
    opts
    (fun x -> pp_es := x :: !pp_es)

let pp_es = List.rev !pp_es

let () =
  let module V = SymbConstant in
  (match !arch with
  | X86 ->
      let module M = Make(X86Compile.Make(V)) in
      M.zyva
  | PPC ->
      let module M = Make(PPCCompile.Make(V)) in
      M.zyva
  | ARM -> Warn.fatal "ARM not implemented yet")
    !name pp_es

