open Printf

let opt = ref AutoOpt.default

let parse_arg d p s = match p s with
| Some m -> m
| None -> raise (Arg.Bad d)

let prog = Sys.argv.(0)

let restart = ref false
open AutoOpt

let read_conf name = opt := AutoLex.conf !opt name

let speclist = 
  ("-v", Arg.Unit (fun () -> opt := incr_verbose !opt),"be verbose")::
  ("-version", Arg.Unit (fun () -> print_endline Version.version ; exit 0),
   " show version number and exit")::
  ("-arch",
   Arg.String (fun s -> opt := set_arch s !opt),
   sprintf "<X86|PPC> architecture (default %s)"
     (Misc.pp_arch default.arch))::
  ("-mode",
   Arg.String (fun s -> opt := set_mode s !opt),
   sprintf "<conform|explo> main mode (default %s)"
     (pp_mode default.mode))::
  ("-nprocs",
   Arg.Int (fun i -> opt := set_nprocs i !opt),
   sprintf "<n> generate tests up to <n> processors (defaults: X86=%i, PPC=%i)"
     (get_nprocs Misc.X86 default) (get_nprocs Misc.PPC default))::
  ("-restart", Arg.Set restart, "restart the experiment in hand")::
  []

let main () =
  Arg.parse speclist read_conf
   (sprintf "Usage: %s [configfile]" prog);
  if !restart then begin
    let name = "." in
    let ropt,c =
      Misc.input_protect_gen
        open_in_bin
        (fun chan ->
          (Marshal.from_channel chan : AutoOpt.t * AutoRun.ckpt))
        (Filename.concat name AutoOpt.ckpt_name) in
    let opt = { ropt  with verbose = !opt.verbose ; } in
    let m = AutoConf.mk_config opt in
    let module C = (val m : AutoConf.S) in
    let module M = AutoRun.Make(C) in
    match C.mode with
    | Explo -> M.restore_explo c
    | Conform -> M.restore_conform c
  end else begin
    let m = AutoConf.mk_config !opt in
    let module C = (val m : AutoConf.S) in
    let module M = AutoRun.Make(C) in
    match C.mode with
    | Explo -> M.go_explo ()
    | Conform -> M.go_conform ()
  end

let () =
  try main ()
  with Misc.Fatal msg ->
    eprintf "Fatal error: %s\n%!" msg ;
    exit 1

