(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

(* Beware, this file is litmus/diy common, change with caution *)

open Printf

let tmp_dir = ref None

let mk_temp_dir () =
  let name = Filename.temp_file  "dir" ".tmp" in
  MySys.mkdir name ;
  name

let get_dir () = match !tmp_dir with
| None -> 
    let name =  mk_temp_dir () in
    tmp_dir := Some name ;
    name
| Some dir -> dir

let outname name = Filename.concat (get_dir ()) name

let tar archive =
  let dir = get_dir () in
  MySys.exec_stdout
    (sprintf "( cd %s && tar cf - . ) > %s && /bin/rm -rf %s"
      dir archive dir)

let tar_as_dir name archive =
  let dir = get_dir () 
  and dir2 = mk_temp_dir () in
  let new_dir = Filename.concat  dir2 name in
  MySys.exec_stdout (sprintf "mv %s %s" dir new_dir) ;
  MySys.exec_stdout (sprintf "( cd %s && tar cf - %s ) > %s" dir2 name archive) ;
  MySys.exec_stdout (sprintf "/bin/rm -rf %s %s" dir dir2)


let tar_structured_dir tarname topdir structures =
  let dir = get_dir () 
  and dir2 = mk_temp_dir () in
  let new_dir = Filename.concat  dir2 topdir in
  MySys.exec_stdout (sprintf "mv %s %s" dir new_dir) ;
  List.iter 
    (fun (subdir,files) -> 
       let new_subdir = Filename.concat new_dir subdir in
       MySys.mkdir new_subdir;
       List.iter 
          (fun fname -> 
               let old_fname = Filename.concat new_dir fname in
               let new_fname = Filename.concat new_subdir fname in
               MySys.exec_stdout (sprintf "mv %s %s" old_fname new_fname))
           files)
     structures; 
  MySys.exec_stdout (sprintf "( cd %s && tar cf - %s ) > %s" dir2 topdir tarname) ;
  MySys.exec_stdout (sprintf "/bin/rm -rf %s %s" dir dir2)
