(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

(***************************************)
(* Facility for building a tar archive *)
(***************************************)

(*
- To output a file in the tar archive, open it as (outname name)
- Build the  tar archive "a.tar" by calling  tar "a.tar"
- Weaknesses:
   Can be used to build one tar archive only.
   Does not handle subdirectories.
*)

(* Gives actual output name, a temporary directory is created at first call *)
val outname : string -> string

(* Produce final tar archive (and remove temporary directory) *)
val tar : string -> unit

(* 'tar dir archive' Similar, but archive contains top directory 'dir' *)
val tar_as_dir : string -> string -> unit

(* 'tar structured dir archive' Similar, but for structured directory:

   tar_structured_dir tarname topdir [subdir,filename list]

   produces an archive tarname, 
   top directory 'topdir', and 
   places filename in subdir *)
val tar_structured_dir : string -> string -> (string * string list) list -> unit

